// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aocTests/Part1TestContext.h>
#include <aocTests/Part2TestContext.h>

#include <aoc/Solver.h>
#include <aoc/SolverEngine.h>
#include <aoc/HistorianHysteria.h>

#include <catch2/catch_test_macros.hpp>

#define REQUIRE_MESSAGE(cond, msg) if (!(cond)) { INFO(msg); REQUIRE(cond); }

class HistorianHysteriaTests
{
public:
    Part1TestContext part1TestContext_{ getInputPaths() };
    Part2TestContext part2TestContext_{ getInputPaths() };
    Part1TestContext part1ExampleTestContext_{ getExampleInputPaths() };
    Part2TestContext part2ExampleTestContext_{ getExampleInputPaths() };

    void runTest(Solver &solver, const long long int expected, const TestContext &context)
    {
        SolverEngine solverEngine{context.getInputPaths()};
        solverEngine.run(solver);

        REQUIRE(expected == context.getResult(solver));
    }

private:
    std::vector<std::string> getInputPaths() const
    {
        return std::vector<std::string>{ "data", "../../../data", "../../../../data" };
    }

    std::vector<std::string> getExampleInputPaths() const
    {
        return std::vector<std::string>{ "data/examples", "../../../data/examples", "../../../../data/examples" };
    }
};

TEST_CASE( "[HistorianHysteriaTests]" )
{
    HistorianHysteriaTests tests;
    SECTION( "FullData1" ) {
        tests.runTest(*std::make_unique<HistorianHysteria>(), 2176849, tests.part1TestContext_);
    }
    SECTION( "FullData2" ) {
        tests.runTest(*std::make_unique<HistorianHysteria>(), 23384288, tests.part2TestContext_);
    }
    SECTION( "ExampleData1" ) {
        tests.runTest(*std::make_unique<HistorianHysteria>(), 11, tests.part1ExampleTestContext_);
    }
    SECTION( "ExampleData2" ) {
        tests.runTest(*std::make_unique<HistorianHysteria>(), 31, tests.part2ExampleTestContext_);
    }
}
